@@ -2,9 +2,13 @@ import sys
 import copy
 import progressbar
 import numpy as np
+import scipy.sparse as sp
 from heapq import heappush, heappushpop
 from itertools import count
 from sklearn.base import BaseEstimator
+import logging
+
+logger = logging.getLogger(__name__)
 
 try:
     import cupy
@@ -21,23 +25,7 @@ except ImportError:
 class LBM(BaseEstimator):
     """
     LBM with distribution.
-
-    Attributes
-    ----------
-    max_iter : int
-        Maximum number of EM iterations
-    n_init : int
-        Number of initializations that will be run for n_iter_early_stop EM iterations.
-    n_init_total_run : int
-        Number of the n_init best initializations that will be run until convergence.
-    nb_iter_early_stop : int
-        Number of EM iterations to used to run the n_init initializations.
-    rtol : float, default 1e-10
-        The relative tolerance parameter (see Notes).
-    atol : float, default 1e-4
-        The absolute tolerance parameter (see Notes).
-    verbosity : int
-        Degree of verbosity. Scale from 0 (no message displayed) to 3.
+    The class implements the random initialisation strategy.
 
     Notes
     -----
@@ -45,6 +33,20 @@ class LBM(BaseEstimator):
     new_loglikelihood - old_loglikelihood <=
     (`atol` + `rtol` * abs(new_loglikelihood)). The convergence is checked
     every 10 EM steps.
+
+
+    Examples
+    --------
+    >>> from sparsebm import LBM
+    >>> model = LBM(
+    ...     nb_row_clusters=4,
+    ...     nb_column_clusters=4,
+    ...     n_init=100,
+    ...     n_iter_early_stop=10,
+    ...     n_init_total_run=5,
+    ...     verbosity=1,
+    ... )
+    >>> model.fit(graph)
     """
 
     def __init__(
@@ -194,9 +196,8 @@ class LBM(BaseEstimator):
         ):
             self.gpu_number = None
             self.use_gpu = False
-            print(
-                "GPU not used as cupy library seems not to be installed or CUDA is not available",
-                file=sys.stderr,
+            logger.warning(
+                "GPU not used as cupy library seems not to be installed or CUDA is not available"
             )
 
         if (
@@ -213,7 +214,7 @@ class LBM(BaseEstimator):
                 free_idx = GPUtil.getAvailable("memory", limit=10)
                 if not free_idx:
                     self.use_gpu = False
-                    print("GPU not used as no gpu is free", file=sys.stderr)
+                    logger.warning("GPU not used as no gpu is free")
                 else:
                     self._np = cupy
                     self._cupyx = cupyx
@@ -257,12 +258,21 @@ class LBM(BaseEstimator):
         n1, n2 = X.shape
         self._nb_rows = n1
         self._nb_cols = n2
-        indices_ones = self._np.asarray(list(X.nonzero()))
+        X = sp.csr_matrix(X)
+        if self.use_gpu:
+            X = self._cupyx.scipy.sparse.csr_matrix(X.astype(float))
+            X_coo = X.tocoo()
+            indices_ones = [X_coo.row, X_coo.col]
+        else:
+            indices_ones = list(X.nonzero())
+
         try:
             # Initialize and start to run each for a while.
 
             if self.verbosity > 0:
-                print("---------- START RANDOM INITIALIZATIONS ---------- ")
+                logger.info(
+                    "---------- START RANDOM INITIALIZATIONS ---------- "
+                )
                 bar = progressbar.ProgressBar(
                     max_value=self.n_init,
                     widgets=[
@@ -293,6 +303,7 @@ class LBM(BaseEstimator):
                     tau_1,
                     tau_2,
                 ) = self._fit_single(
+                    X,
                     indices_ones,
                     n1,
                     n2,
@@ -314,7 +325,7 @@ class LBM(BaseEstimator):
                     heappushpop(best_inits, calculation_result)
             if self.verbosity > 0:
                 bar.finish()
-                print(
+                logger.info(
                     "---------- START TRAINING BEST INITIALIZATIONS ---------- "
                 )
                 bar = progressbar.ProgressBar(
@@ -353,6 +364,7 @@ class LBM(BaseEstimator):
                     tau_1,
                     tau_2,
                 ) = self._fit_single(
+                    X,
                     indices_ones,
                     n1,
                     n2,
@@ -377,6 +389,7 @@ class LBM(BaseEstimator):
 
     def _fit_single(
         self,
+        X,
         indices_ones,
         n1,
         n2,
@@ -389,6 +402,8 @@ class LBM(BaseEstimator):
 
         Parameters
         ----------
+        X : scipy.sparse.csr_matrix, shape=(n1, n2)
+            Matrix to be analyzed
         indices_ones : Non zero indices of the data matrix.
         n1 : Number of rows in the data matrix.
         n2 : Number of columns in the data matrix.
@@ -418,11 +433,7 @@ class LBM(BaseEstimator):
                 (pi, alpha_1, alpha_2, tau_1, tau_2) = init_params
         else:
             alpha_1, alpha_2, tau_1, tau_2, pi = self._init_LBM_random(
-                n1,
-                n2,
-                self.n_row_clusters,
-                self.n_column_clusters,
-                len(indices_ones),
+                n1, n2, self.n_row_clusters, self.n_column_clusters, X.nnz
             )
 
         # Repeat EM step until convergence.
@@ -439,19 +450,20 @@ class LBM(BaseEstimator):
                 if (ll - old_ll) < (self.atol + self.rtol * self._np.abs(ll)):
                     success = True
                     break
-                if self.verbosity > 2:
 
-                    print(
-                        f"\t EM Iter: {iteration:5d}  \t  log-like:{ll.get() if self.use_gpu else ll:.4f} \t diff:{self._np.abs(old_ll - ll).get() if self.use_gpu else self._np.abs(old_ll - ll):.6f}"
-                    )
+                log_txt = f"\t EM Iter: {iteration:5d}  \t  log-like:{ll.get() if self.use_gpu else ll:.4f} \t diff:{self._np.abs(old_ll - ll).get() if self.use_gpu else self._np.abs(old_ll - ll):.6f}"
+                if self.verbosity > 1:
+                    logger.info(log_txt)
+                else:
+                    logger.debug(log_txt)
                 old_ll = ll
             pi, alpha_1, alpha_2, tau_1, tau_2 = self._step_EM(
-                indices_ones, pi, alpha_1, alpha_2, tau_1, tau_2, n1, n2
+                X, indices_ones, pi, alpha_1, alpha_2, tau_1, tau_2, n1, n2
             )
         else:
             success = True
         if self.verbosity > 1 and run_number:
-            print(
+            logger.info(
                 f"Run {run_number:3d} / {self.n_init:3d} \t success : {success} \t log-like: {ll.get()  if self.use_gpu else ll:.4f} \t nb_iter: {iteration:5d}"
             )
 
@@ -467,12 +479,14 @@ class LBM(BaseEstimator):
         return success, ll, pi, alpha_1, alpha_2, tau_1, tau_2
 
     def _step_EM(
-        self, indices_ones, pi, alpha_1, alpha_2, tau_1, tau_2, n1, n2
+        self, X, indices_ones, pi, alpha_1, alpha_2, tau_1, tau_2, n1, n2
     ):
         """Realize EM step. Update both variationnal and model parameters.
 
         Parameters
         ----------
+        X : scipy.sparse.csr_matrix, shape=(n1, n2)
+            Matrix to be analyzed
         indices_ones : Non zero indices of the data matrix.
         pi : Connection probability matrix between row and column groups.
         alpha_1 : Row group model parameters.
@@ -488,16 +502,8 @@ class LBM(BaseEstimator):
         nq, nl = self.n_row_clusters, self.n_column_clusters
 
         ########################## E-step  ##########################
-
-        # Precomputations needed.
-        u = self._np.zeros((n1, nl))
-        v = self._np.zeros((n2, nq))
-        if self.use_gpu:
-            self._cupyx.scatter_add(u, indices_ones[0], tau_2[indices_ones[1]])
-            self._cupyx.scatter_add(v, indices_ones[1], tau_1[indices_ones[0]])
-        else:
-            self._np.add.at(u, indices_ones[0], tau_2[indices_ones[1]])
-            self._np.add.at(v, indices_ones[1], tau_1[indices_ones[0]])
+        u = X.dot(tau_2)  # Shape is (n1,nl)
+        v = X.T.dot(tau_1)  # Shape is (n2,nq)
 
         # Update of tau_1 with sparsity trick.
         l_tau_1 = (
@@ -599,6 +605,9 @@ class LBM(BaseEstimator):
         tau_2[tau_2 < eps_2] = eps_2
         tau_2 /= tau_2.sum(axis=1).reshape(n2, 1)  # Re-Normalize.
         pi = self._np.random.uniform(0, 1e-7, (nq, nl))
+        pi = self._np.random.uniform(
+            0.2 * nb_ones / (n1 * n2), 2 * nb_ones / (n1 * n2), (nq, nl)
+        )
         return (alpha_1.flatten(), alpha_2.flatten(), tau_1, tau_2, pi)
 
     def __repr__(self):
