@@ -9,6 +9,21 @@ from .utils import (
 )
 from typing import Any, Tuple, Union, Optional
 from scipy.sparse import spmatrix
+import scipy.sparse as sp
+import logging
+
+logger = logging.getLogger(__name__)
+
+try:
+    import cupy
+    import cupyx
+    import GPUtil
+
+    _CUPY_INSTALLED = True
+    _DEFAULT_USE_GPU = True
+except ImportError:
+    _CUPY_INSTALLED = False
+    _DEFAULT_USE_GPU = False
 
 
 class ModelSelection:
@@ -16,6 +31,14 @@ class ModelSelection:
     Explore and select the optimal number of classes for the LBM or SBM model.
     The best model is chosen according to the Integrated Completed Likelihood.
     A strategy of merging and splitting classes to produce good initializations is used.
+
+    Examples
+    --------
+    >>> lbm_model_selection = ModelSelection(
+    ...     model_type="LBM",
+    ...     plot=True,
+    ... )
+    >>> lbm_selected = lbm_model_selection.fit(graph)
     """
 
     def __init__(
@@ -45,7 +68,7 @@ class ModelSelection:
             raise Exception("model_type parameter must be 'SBM' or 'LBM'")
 
         self._model_type = model_type
-        self._use_gpu = use_gpu
+        self._use_gpu = use_gpu if (use_gpu and _CUPY_INSTALLED) else False
         self._gpu_index = gpu_index
         self._plot = plot
         self._figure = plt.subplots(1) if plot else None
@@ -92,6 +115,11 @@ class ModelSelection:
             np.asarray(graph.sum(1)).squeeze(),
             np.asarray(graph.sum(0)).squeeze(),
         )
+
+        self._X = sp.csr_matrix(graph)
+        if self._use_gpu:
+            self._X = cupyx.scipy.sparse.csr_matrix(self._X.astype(float))
+
         # Instantiate and training first model.
         if self._model_type == "LBM":
             model = LBM(
@@ -140,13 +168,13 @@ class ModelSelection:
                     for m in self.model_explored.values()
                 ]
             ):
-                print("Spliting")
+                logger.info("Spliting")
                 self.model_explored = self._explore_strategy(strategy="split")
-                print("Merging")
+                logger.info("Merging")
                 self.model_explored = self._explore_strategy(strategy="merge")
                 best_iter_model = self.selected_model
                 best_icl.append(best_iter_model.get_ICL())
-                print("Best icl is {:.4f}".format(best_icl[-1]))
+                logger.info("Best icl is {:.4f}".format(best_icl[-1]))
                 if len(best_icl) > 3 and best_icl[-3] == best_icl[-1]:
                     break
         except KeyboardInterrupt:
@@ -245,14 +273,14 @@ class ModelSelection:
                             else best_model["model"].n_clusters
                         )
 
-                    print(
+                    logger.info(
                         "\t Already explored models from {} classes".format(
                             nnq
                         )
                     )
                     continue
             model_flag[flag_key] = True
-            print("\t Explore models from {} classes".format(nnq))
+            logger.info("\t Explore models from {} classes".format(nnq))
 
             if self._model_type == "LBM":
                 # Explore all models derived from the strategy on the rows.
@@ -432,6 +460,7 @@ class ModelSelection:
         for ic, m in models[:5]:
             if self._model_type == "LBM":
                 m._fit_single(
+                    self._X,
                     self._indices_ones,
                     self.graph.shape[0],
                     self.graph.shape[1],
@@ -441,6 +470,7 @@ class ModelSelection:
                 )
             else:
                 m._fit_single(
+                    self._X,
                     self._indices_ones,
                     self.graph.shape[0],
                     init_params=True,
@@ -453,6 +483,7 @@ class ModelSelection:
         # The best model is trained until convergence.
         if self._model_type == "LBM":
             models[0][1]._fit_single(
+                self._X,
                 self._indices_ones,
                 self.graph.shape[0],
                 self.graph.shape[1],
@@ -461,6 +492,7 @@ class ModelSelection:
             )
         else:
             models[0][1]._fit_single(
+                self._X,
                 self._indices_ones,
                 self.graph.shape[0],
                 init_params=True,
@@ -520,9 +552,9 @@ def _plot_merge_split_graph(
         ax.set_xlim((0, max(10, max(nqs), max(nqs_prev))))
         ax.set_ylim((0, max(10, max(nls), max(nls_prev))))
         if strategy == "merge":
-            ax.set_title("Merging strategy")
+            ax.set_title("Merging step")
         else:
-            ax.set_title("Spliting strategy")
+            ax.set_title("Spliting step")
         ax.set_ylabel("Number of column groups")
         ax.set_xlabel("Number of row groups")
         ax.grid()
@@ -533,7 +565,7 @@ def _plot_merge_split_graph(
             s=100,
             c="grey",
             marker="+",
-            label="Models explored during previous strategy",
+            label="Models explored during previous step",
         )
         ax.scatter(
             nqs,
@@ -541,30 +573,7 @@ def _plot_merge_split_graph(
             s=70,
             c="orange",
             marker="o",
-            label="Models explored during current strategy",
-        )
-        ax.scatter(
-            [c[0] for c in currently_explored_nqnl],
-            [c[1] for c in currently_explored_nqnl],
-            s=30,
-            c="red",
-            marker="o",
-            label="Models currently explored",
-        )
-        ax.scatter(
-            [best_model_current_strategy["model"].n_row_clusters],
-            [best_model_current_strategy["model"].n_column_clusters],
-            s=80,
-            c="green",
-            marker="o",
-            label="Optimal model of the current strategy",
-        )
-        ax.annotate(
-            str(round(model_selection.selected_model.get_ICL(), 2)),
-            xy=(
-                model_selection.selected_model.n_row_clusters - 0.5,
-                model_selection.selected_model.n_column_clusters + 0.25,
-            ),
+            label="Models explored at current step",
         )
         ax.scatter(
             [model_selection.selected_model.n_row_clusters],
@@ -572,7 +581,7 @@ def _plot_merge_split_graph(
             s=120,
             c="black",
             marker="*",
-            label="Optimal model from all strategies",
+            label="Current optimal model",
         )
         ax.annotate(
             str(round(model_selection.selected_model.get_ICL(), 2)),
@@ -595,9 +604,9 @@ def _plot_merge_split_graph(
 
         ax.set_xlim((0, max(10, max(nqs), max(nqs_prev))))
         if strategy == "merge":
-            ax.set_title("Merging strategy")
+            ax.set_title("Merging step")
         else:
-            ax.set_title("Spliting strategy")
+            ax.set_title("Spliting step")
         ax.set_ylabel("ICL")
         ax.set_xlabel("Number of row groups")
         ax.grid()
@@ -607,7 +616,7 @@ def _plot_merge_split_graph(
             s=100,
             c="grey",
             marker="+",
-            label="Models explored during previous strategy",
+            label="Models explored during previous step",
         )
         ax.scatter(
             nqs,
@@ -615,15 +624,7 @@ def _plot_merge_split_graph(
             s=70,
             c="orange",
             marker="o",
-            label="Models explored during current strategy",
-        )
-        ax.scatter(
-            [best_model_current_strategy["model"].n_clusters],
-            [best_model_current_strategy["icl"]],
-            s=80,
-            c="green",
-            marker="o",
-            label="Optimal model of the current strategy",
+            label="Models explored at current step",
         )
         ax.scatter(
             [model_selection.selected_model.n_clusters],
@@ -631,7 +632,7 @@ def _plot_merge_split_graph(
             s=120,
             c="black",
             marker="*",
-            label="Optimal model from all strategies",
+            label="Current optimal model",
         )
     ax.legend()
     plt.pause(0.01)
