@@ -2,10 +2,14 @@ import sys
 import copy
 import progressbar
 import numpy as np
+import scipy.sparse as sp
 from heapq import heappush, heappushpop
 from itertools import count
 from sklearn.utils.estimator_checks import check_estimator
 from sklearn.base import BaseEstimator
+import logging
+
+logger = logging.getLogger(__name__)
 
 try:
     import cupy
@@ -21,31 +25,32 @@ except ImportError:
 
 class SBM(BaseEstimator):
     """
-    SBM with bernoulli distribution.
-
-    Attributes
-    ----------
-    max_iter : int
-        Maximum number of EM iterations
-    n_init : int
-        Number of initializations that will be run for n_iter_early_stop EM iterations.
-    n_init_total_run : int
-        Number of the n_init best initializations that will be run until convergence.
-    n_iter_early_stop : int
-        Number of EM iterations to used to run the n_init initializations.
-    rtol : float, default 1e-10
-        The relative tolerance parameter (see Notes).
-    atol : float, default 1e-4
-        The absolute tolerance parameter (see Notes).
-    verbosity : int
-        Degree of verbosity. Scale from 0 (no message displayed) to 3.
+    SBM with bernoulli distribution. The class implements the random
+    initialisation strategy.
 
     Notes
     -----
     Convergence of the EM algorithm is declared when
     new_loglikelihood - old_loglikelihood <=
     (`atol` + `rtol` * absolute(new_loglikelihood)). The convergence is checked
-    every 10 EM steps.
+    every 5 EM steps.
+
+    Examples
+    --------
+    >>> from sparsebm import SBM
+    >>> model = SBM(
+    ...     n_clusters=4,
+    ...     max_iter=10000,
+    ...     n_init=100,
+    ...     n_init_total_run=10,
+    ...     n_iter_early_stop=10,
+    ...     rtol=1e-10,
+    ...     atol=1e-4,
+    ...     verbosity=1,
+    ...     use_gpu=True,
+    ...     gpu_index=0,
+    ... )
+    >>> model.fit(graph)
     """
 
     def __init__(
@@ -137,9 +142,8 @@ class SBM(BaseEstimator):
         ):
             self.gpu_number = None
             self.use_gpu = False
-            print(
-                "GPU not used as cupy library seems not to be installed or CUDA is not available",
-                file=sys.stderr,
+            logger.warning(
+                "GPU not used as cupy library seems not to be installed or CUDA is not available"
             )
 
         if (
@@ -156,7 +160,7 @@ class SBM(BaseEstimator):
                 free_idx = GPUtil.getAvailable("memory", limit=10)
                 if not free_idx:
                     self.use_gpu = False
-                    print("GPU not used as no gpu is free", file=sys.stderr)
+                    logger.warning("GPU not used as no gpu is free")
                 else:
                     self._np = cupy
                     self._cupyx = cupyx
@@ -232,12 +236,21 @@ class SBM(BaseEstimator):
         n, n2 = X.shape
         assert n == n2, "Entry matrix is not squared"
         self._nb_rows = n
-        indices_ones = self._np.asarray(list(X.nonzero()))
+        X = sp.csr_matrix(X)
+        if self.use_gpu:
+            X = self._cupyx.scipy.sparse.csr_matrix(X.astype(float))
+            X_coo = X.tocoo()
+            indices_ones = [X_coo.row, X_coo.col]
+        else:
+            indices_ones = list(X.nonzero())
+
         try:
             # Initialize and start to run each for a while.
 
             if self.verbosity > 0:
-                print("---------- START RANDOM INITIALIZATIONS ---------- ")
+                logger.info(
+                    "---------- START RANDOM INITIALIZATIONS ---------- "
+                )
                 bar = progressbar.ProgressBar(
                     max_value=self.n_init,
                     widgets=[
@@ -260,6 +273,7 @@ class SBM(BaseEstimator):
                 if self.verbosity > 0:
                     bar.update(run_number)
                 (success, ll, pi, alpha, tau) = self._fit_single(
+                    X,
                     indices_ones,
                     n,
                     early_stop=self.n_iter_early_stop,
@@ -272,7 +286,7 @@ class SBM(BaseEstimator):
                     heappushpop(best_inits, calculation_result)
             if self.verbosity > 0:
                 bar.finish()
-                print(
+                logger.info(
                     "---------- START TRAINING BEST INITIALIZATIONS ---------- "
                 )
                 bar = progressbar.ProgressBar(
@@ -298,6 +312,7 @@ class SBM(BaseEstimator):
                 (pi, alpha, tau) = (init[2], init[3], init[4])
 
                 (success, ll, pi, alpha, tau) = self._fit_single(
+                    X,
                     indices_ones,
                     n,
                     init_params=(pi, alpha, tau),
@@ -319,6 +334,7 @@ class SBM(BaseEstimator):
 
     def _fit_single(
         self,
+        X,
         indices_ones,
         n,
         early_stop=None,
@@ -330,8 +346,10 @@ class SBM(BaseEstimator):
 
         Parameters
         ----------
+        X : scipy.sparse.csr_matrix, shape=(n,n)
+            Matrix to be analyzed
         indices_ones : Non zero indices of the data matrix.
-        n1 : Number of rows in the data matrix.
+        n : Number of rows in the data matrix.
         """
         old_ll = -self._np.inf
         success = False
@@ -353,10 +371,7 @@ class SBM(BaseEstimator):
             else:
                 (pi, alpha, tau) = init_params
         else:
-            alpha, tau, pi = self._init_SBM_random(
-                n, self.n_clusters, len(indices_ones)
-            )
-
+            alpha, tau, pi = self._init_SBM_random(n, self.n_clusters, X.nnz)
         # Repeat EM step until convergence.
         for iteration in range(self.max_iter):
             if early_stop and iteration >= early_stop:
@@ -367,17 +382,18 @@ class SBM(BaseEstimator):
                 if (ll - old_ll) < (self.atol + self.rtol * self._np.abs(ll)):
                     success = True
                     break
-                if self.verbosity > 2:
 
-                    print(
-                        f"\t EM Iter: {iteration:5d}  \t  log-like:{ll.get() if self.use_gpu else ll:.4f} \t diff:{self._np.abs(old_ll - ll).get() if self.use_gpu else self._np.abs(old_ll - ll):.6f}"
-                    )
+                log_txt = f"\t EM Iter: {iteration:5d}  \t  log-like:{ll.get() if self.use_gpu else ll:.4f} \t diff:{self._np.abs(old_ll - ll).get() if self.use_gpu else self._np.abs(old_ll - ll):.6f}"
+                if self.verbosity > 1:
+                    logger.info(log_txt)
+                else:
+                    logger.debug(log_txt)
                 old_ll = ll
-            pi, alpha, tau = self._step_EM(indices_ones, pi, alpha, tau, n)
+            pi, alpha, tau = self._step_EM(X, indices_ones, pi, alpha, tau, n)
         else:
             success = True
         if self.verbosity > 1 and run_number:
-            print(
+            logger.info(
                 f"Run {run_number:3d} / {self.n_init:3d} \t success : {success} \t log-like: {ll.get()  if self.use_gpu else ll:.4f} \t nb_iter: {iteration:5d}"
             )
 
@@ -390,11 +406,13 @@ class SBM(BaseEstimator):
 
         return success, ll, pi, alpha, tau
 
-    def _step_EM(self, indices_ones, pi, alpha, tau, n1):
+    def _step_EM(self, X, indices_ones, pi, alpha, tau, n1):
         """Realize EM step. Update both variationnal and model parameters.
 
         Parameters
         ----------
+        X : scipy.sparse.csr_matrix, shape=(n1, n1)
+            Matrix to be analyzed
         indices_ones : Non zero indices of the data matrix.
         pi : Connection probability matrix between row and column groups.
         alpha : Group model parameters.
@@ -405,13 +423,7 @@ class SBM(BaseEstimator):
         nq = self.n_clusters
 
         ########################## E-step  ##########################
-
-        # Precomputations needed.
-        u = self._np.zeros((n1, nq))
-        if self.use_gpu:
-            self._cupyx.scatter_add(u, indices_ones[0], tau[indices_ones[1]])
-        else:
-            self._np.add.at(u, indices_ones[0], tau[indices_ones[1]])
+        u = X.dot(tau)
 
         # Update of tau_1 with sparsity trick.
         l_tau = (
@@ -420,9 +432,20 @@ class SBM(BaseEstimator):
                 * (self._np.log(pi) - self._np.log(1 - pi)).reshape(1, nq, nq)
             ).sum(2)
             + self._np.log(alpha.reshape(1, nq))
-            + tau.sum(0) @ np.log(1 - pi.T)
-            - tau @ np.log(1 - pi.T)
+            + tau.sum(0) @ np.log(1 - pi)
+            - tau @ np.log(1 - pi)
         )
+        if self.symmetric == False:
+            X_t = X.transpose()
+            v = X_t.dot(tau)
+            l_tau += (
+                (
+                    (v.reshape(n1, 1, nq))
+                    * (np.log(pi.T) - np.log(1 - pi.T)).reshape(1, nq, nq)
+                ).sum(2)
+                + tau.sum(0) @ np.log(1 - pi.T)
+                - tau @ np.log(1 - pi.T)
+            )
 
         # For computationnal stability reasons 1.
         l_tau -= l_tau.max(axis=1).reshape(n1, 1)
